<?php

namespace Drupal\pagedesigner_block_adaptable\Plugin\pagedesigner_block_adaptable\Filter;

/**
 * Process entities of type "numeric".
 *
 * @PagedesignerFilter(
 *   id = "pagedesigner_filter_numeric",
 *   name = @Translation("Numeric filter"),
 *   types = {
 *     "numeric",
 *   },
 * )
 */
class Numeric extends EntityFilter {

  /**
   * {@inheritDoc}
   */
  public function build(array &$filterDefinition) : array {
    $this->adjustFilter($filterDefinition);
    $renderArray = [
      'label' => $filterDefinition['field'],
      'type' => 'text',
    ];
    // Handle reference fields (@deprecated in project:3.0.0).
    if (substr($filterDefinition['field'], -10) == '_target_id' && substr($filterDefinition['field'], 0, 6) == 'field_') {
      $bundle = substr($filterDefinition['field'], 6, -10);
      $items = \Drupal::entityTypeManager()->getStorage('node')->loadByProperties([
        'type' => $bundle,
      ]);
      $options = [];
      $values = [];
      foreach ($items as $item) {
        if ($item != NULL) {
          $options[$item->id()] = $item->label();
        }
      }
      $renderArray = [
        'description' => t('Choose @label', ['@label' => $filterDefinition['expose']['label']]),
        'label' => $filterDefinition['expose']['label'],
        'options' => $options,
        'type' => $filterDefinition['pagedesigner_trait_type'],
        'name' => $filterDefinition['id']['label'],
        'value' => $values,
      ];
    }
    else {
      $renderArray = parent::build($filterDefinition);
    }
    return $renderArray;
  }

  /**
   * {@inheritDoc}
   */
  public function adjustFilter(array &$filterDefinition) {
    if (empty($filterDefinition['pagedesigner_trait_type'])) {
      $filterDefinition['pagedesigner_trait_type'] = 'select';
    }
  }

}
